package com.xkrs.dao;

import com.xkrs.model.entity.ShanDongFirePointEntity;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
public interface ShanDongFirePointDao extends JpaRepository<ShanDongFirePointEntity, Long>, JpaSpecificationExecutor<ShanDongFirePointEntity> {

    /**
     * 根据火点编码查询火点信息
     *
     * @param fireCode
     * @return
     */
    ShanDongFirePointEntity findByFireCode(String fireCode);

    @Query(value = "SELECT * FROM fire_point_shandong WHERE longitude = ?1 AND latitude = ?2 AND street_code = ?3 AND satellite_time = ?4 AND satellite_type = ?5 AND land_type = ?6", nativeQuery = true)
    List<ShanDongFirePointEntity> findDuplicatedData(double longitude, double latitude, String streetCode, String satelliteTime, String satelliteType, String landType);

    /**
     * 在山东临时火点表中根据火点编码修改审核状态
     */
    @Modifying(clearAutomatically = true)
    @Query(value = "UPDATE fire_point_shandong SET verify_state = ?2 WHERE fire_code = ?1", nativeQuery = true)
    void updateVerifyStateByFireCode(String fireCode, String verifyState);

}
